<?php
/*
Plugin Name: Buddypress Bulk Group Importer
Plugin URI: 
Description: Creating groups is important as they reduce the load on administrators by allowing them bulk group of users at one go, instantly. This wordpress plugin allows imports BuddyPress groups with their settings from a CSV file. Prepare CSV file, select bulk settings if needed and then click import. That's all, enjoy.

Bulk Group Creation
•	Bulk Create Parent member group with unlimited members. 
•	Give a brief description to individual group in CSV . 
•	Import CSV file and create groups in patch.
•	Send Invite to Group and Group members from settings
•	Single instance Creation of Groups.

Simply install the plugin and create a csv file with two columns i.e group name and group description. The plugin will then allow you to import csv file and create bulk group. Created group members will receive notification/invite to join group on your site as well. 

Creating and managing bulk BuddyPress groups for your Wordpress site has never been easier!

Version: 1.0 
Author: NwareIndia
Author URI: http://www.nwareindia.com/
License: GPL2
*/

// Is ABSPATH defined?
if ( ! defined( 'ABSPATH' ) ) exit;

$pluginfoldername = basename(__DIR__);
$rootpluginfoldername = basename(basename(__DIR__));
define('Bp_Bulk_Plugin_Name', $pluginfoldername);
define('Plugin_Dir', $rootpluginfoldername);

if(!class_exists('Bp_Bulk_Group_Importer'))
{
    class Bp_Bulk_Group_Importer  
    {
        /**
         * Construct the plugin object
         */
        public function __construct()
        {
            // register actions
            add_action('admin_init', array(&$this, 'admin_init'));
			add_action('admin_menu', array(&$this, 'add_menu'));
			#Using registered $page handle to hook stylesheet loading      		
       		add_action( 'admin_enqueue_scripts', array(&$this, 'register_plugin_styles' ) );
       		add_action( 'admin_enqueue_scripts', array(&$this, 'register_plugin_scripts' ) );       		
		} 

		/**
         * Activate the plugin
        */
        public static function activate()
        {
        	ob_start();
           	update_option('bp_bulk_group_version', '1.0' );
           	global $wpdb;		  	
			$charset_collate = $wpdb->get_charset_collate();
			$table_group = $wpdb->prefix . 'bp_bulk_group';			

			$group = "CREATE TABLE $table_group (
				ID INT NOT NULL AUTO_INCREMENT,
				bp_bulk_name VARCHAR(200) NOT NULL,
				bp_bulk_group VARCHAR(200) NOT NULL,
				created TIMESTAMP NOT NULL,
				count INT(10),
				PRIMARY KEY (ID)
			) $charset_collate;";

			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $group );			
        } 
    
        /**
         * Deactivate the plugin
         */     
        public static function deactivate()
        {
            
        }

        /*
        * uninstall the plugin
        */
        public static function bp_bulk_deinstall()
        {
        	global $wpdb;
	        $table_group = $wpdb->prefix . 'bp_bulk_group';

	        //Delete any options thats stored also?
			$wpdb->query("DROP TABLE IF EXISTS $table_group");
        }

        /**
		 * hook into WP's admin_init action hook
		*/
		public function admin_init()
		{		    
		    // Possibly do additional admin_init tasks
		    if(!current_user_can('manage_options'))
		    {
		        wp_die(__('You do not have sufficient permissions to access this page.'));
		    }
		}

		/*
		* geting all the group
		*/
		public function buddpressgroup()
		{
			if (class_exists('BP_Groups_Group')) {
			    // generate the class here
			    $vgroups = BP_Groups_Group::get(array(
		            'type'=>'alphabetical'
		            )
		        );
		        return $vgroups;
			}else{
				$this->error_show('Buddypress Group plugin is required....!');
				exit();
			}
			
		}

		/*
		* function to update setting option of the buddypress
		*/
		public function update_buddypress_settings_option($post)
		{
			$bp_status 	= sanitize_text_field( $post['bp_group_status']);
			$bp_invite 	= sanitize_text_field( $post['bp_group_invite_status'] );
			$bp_desc 	= sanitize_text_field( $post['bp_group_description'] );
			##
			update_option( 'bp_group_status', $bp_status);
			update_option( 'bp_group_invite_status',  $bp_invite);
			update_option( 'bp_group_description',  $bp_desc);
			$msg = 'Buddypress Setting are saved.....!';
			$this->message_show($msg);
		}

		/**
		* Initialize add group funtion
		*/     
		public function add_group($post,$r)
		{
			//register the settings for this plugin
			global $wpdb;		   
			$table = $wpdb->prefix.'bp_bulk_group';	
			$data = $wpdb->get_results("SELECT * FROM $table WHERE bp_bulk_name='".$post['bp_bulk_name']."' AND bp_bulk_group='".$post['bp_bulk_cat']."'");	   
			/*=======*/
			if($data)
			{
				$msg = 'Group and Subreddit already Exists.....!';
				$this->message_show($msg);
			}else if(!empty($post['bp_bulk_name']) && !empty($post['bp_bulk_cat'])){
				$wpdb->insert($table, array(
					"bp_bulk_name"  => $post['bp_bulk_name'],
					"bp_bulk_group" => $post['bp_bulk_cat'],
					"created"      => date('Y-m-d H:i:s'),
					"count"        => 0			   	
				));  
				if($r == 1){
					$msg = 'Group and Subreddit added sucessfully.....!';
					$this->message_show($msg);
				}
			}
		}

		/*
		* Add group By Csv
		*/
		public function add_group_csv($files)
		{
			if(isset($files['xmlupload'])){
				$upload_dir = wp_upload_dir();
				$basedir    = $upload_dir['basedir'];
				$file 		= $files['xmlupload']['name'];
				$target_file = $basedir.'/'.$files['xmlupload']['name'];
				if (move_uploaded_file($files["xmlupload"]["tmp_name"], $target_file)) 
				{
					$row = 1;
					if (($handle = fopen($target_file, "r")) !== FALSE) 
					{
						while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) 
						{
							$this->bpgi_page_display($data);  
							$num = count($data);
							if($row > 1 )
							{
								$post1 = array();
								$group_id = $this->buddgroupidbyname($data[0]); 
								$post1['bp_bulk_name'] = $data[1]; 
								$post1['bp_bulk_cat'] = $group_id;  
								if(!empty($group_id)){
									$this->add_group($post1,$row-1);
								}
							}
							$row++;
						}
						fclose($handle);
					}				
				} 				
			}
		}

		/*
		* trim group Name
		*/
		public function buddgroupidbyname($name)
		{
			$name = str_replace(" ","-",$name);
			$sname = strtolower($name);
			$vgroups = BP_Groups_Group::group_exists($sname);
	        return $vgroups;
		}
		
		/*
		* add the buddypress group with the csv
		*/		
		public function bpgi_page_display($data) 
		{
		    $notice = '';
		    $errors = '';
		    // setting option
		    $bp_group_status 	= get_option('bp_group_status');
		    $bp_invite_status 	= get_option('bp_group_invite_status');
		    $bp_group_desc 		= get_option('bp_group_description');

		    # start import
		    if (!empty($data)) 
		    {
		        global $bp, $wpdb;
				$group_id 		= $this->buddgroupidbyname($data[0]); 
				$subgroup_id 	= $this->buddgroupidbyname($data[0]); 
	            if($group_id == '')
	            {
					# get form values and validate
					$parent_group_id 		= '';
					$group_status 			= $bp_group_status;
					$group_invite_status 	= $bp_invite_status;
					if (!isset($group_forum)) $group_forum = false;
					if (!isset($group_overwrite)) $group_overwrite = false;
					$group_parent 	= $data[0];
					$group_name 	= $data[0];
					if($data[1] == ''){
						$group_description = $bp_group_desc;
					}else{
						$group_description = $data[1];
					}
					$group_forum 	= 'yes';
					$group_status 	= $bp_group_status;
					$group_invite_status = $bp_invite_status;
					# load CSV file
					# read 1000 lines per run
					$group_count 		= 0;
					# get details from csv file
					$csv_group_parent 	= trim($group_parent);
					$csv_group_name 	= trim($group_name);
					$csv_group_description 	= trim($group_description);
					$csv_group_status 		= trim($group_status);
					$csv_group_forum 		=0;
					$csv_group_invite_status = trim($group_invite_status);
	
					if ($csv_group_forum == 'yes') $csv_group_forum = 1; else $csv_group_forum = 0;
	
					# find parent group ID
					$parent_group_id = groups_get_id(sanitize_title_with_dashes(esc_attr($csv_group_parent)));
	
					# bulk overwrite
					if ($group_overwrite == 'yes') 
					{
						$csv_group_parent 	= $parent_group;
						$csv_group_status 	= $group_status;
						$csv_group_forum 	= $group_forum;
						$csv_group_invite_status = $group_invite_status;
						$parent_group_id 	= $csv_group_parent;
					}
					$group_slug = groups_check_slug(sanitize_title_with_dashes(esc_attr($csv_group_name)));
					
					$args = array (
						'name'          => $csv_group_name,
						'description'   => $csv_group_description,
						'slug'          => $group_slug,
						'status'        => $csv_group_status,
						'enable_forum'  => $csv_group_forum
					);
					$new_group_id = groups_create_group ($args);
	
					# group created successfully
					if (!empty($new_group_id)) 
					{
						# add BP activity
						bp_activity_add (array(
						  'action'            => '<a href="'.bp_loggedin_user_domain('/').'" title="'.bp_get_loggedin_user_username().'">'. bp_get_loggedin_user_username().'</a> created the group <a href="'.site_url().'/groups/'.$group_slug.'/">'.$csv_group_name.'</a>',
						  'component'         => 'groups',
						  'type'              => 'created_group',
						  'primary_link'      => bp_loggedin_user_domain('/'),
						  'user_id'           => bp_loggedin_user_id(),
						  'item_id'           => $new_group_id
						));
	
						# set invite status
						groups_update_groupmeta( $new_group_id, 'invite_status', $csv_group_invite_status );
							
						# set parent group if exist
						if ($parent_group_id>0) {
							$sql = $wpdb->prepare(
								"UPDATE {$bp->groups->table_name} SET parent_id = %d WHERE id = %d",
								$parent_group_id,
								$new_group_id
							);
							$wpdb->query($sql);
						}
	
						$group_count++;
					}
					else {
						$errors[] = 'Cannot create group, probably a temporary mysql error';
					}// else
				}				
				# get currently created groups
				$get_groups = groups_get_groups( array(
				   'populate_extras'   => 'false',
				   'order'             => 'ASC',
				   'orderby'           => 'name')
				);
			}
		}

		/**
		 * message show
		*/
		public function message_show($msg){
			echo '<div class="updated"><br>';
			echo '<div class="alert alert-success alert-dismissible" role="alert">
                <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                <strong>Well done!</strong> '.$msg.'
            </div>';
            echo '</div>';
		}

		/**
		 * message show
		*/
		public function error_show($msg){
			echo '<div class="notice notice-warning"><br>';
			echo '<div class="alert alert-danger alert-dismissible" role="alert">
                <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                <strong>Warning!</strong> '.$msg.'
            </div>';
			echo '</div>';
		}

		/*
		 * add a menu
		*/     
		public function add_menu()
		{		    
		    add_menu_page('Bp Bulk Group Importer', 'Bp Bulk Group Importer', 'manage_options', 'bp-bulk-options', array(&$this, 'bp_bulk_grup_importer'));  			
  			add_submenu_page( 'bp-bulk-options', 'Add Groups', 'Add Groups', 'manage_options', 'bp-bulk-options', array(&$this, 'bp_bulk_grup_importer')); 
  			add_submenu_page( 'bp-bulk-options', 'Buddypress Settings', 'Buddypress Settings', 'manage_options', 'bp-bulk-settings', array(&$this, 'bp_bulk_group_setting')); 			
		} 
		
		/**
		 * Register and enqueue style sheet.
		 */
		public function register_plugin_styles() 
		{
			wp_register_style( 'Bp_Bulk_Group_Importer', plugins_url('/css/bp-bulk.css', __FILE__ ) );
			wp_enqueue_style( 'Bp_Bulk_Group_Importer' );
		}

		/**
	     * Register and enqueue scripts.
	    */
	    public function register_plugin_scripts() 
	    {	        
	        wp_register_script( 'myscript', plugins_url('/js/bp-bulk.js', __FILE__ ), array(), '1.0.0', false);
	        wp_enqueue_script( 'myscript' );

	    }

		/**
		 * Menu Callback
		 */     
		public function bp_bulk_grup_importer()
		{
		    if(!current_user_can('manage_options'))
		    {
		        wp_die(__('You do not have sufficient permissions to access this page.'));
		    }		    
		   
		    // Render the template    		
    		include(sprintf("%s/adds.php", dirname(__FILE__)));    		
		}		

		/*
		* Buddypress Setting page
		*/ 
		public function bp_bulk_group_setting()
		{
			if(!current_user_can('manage_options'))
		    {
		        wp_die(__('You do not have sufficient permissions to access this page.'));
		    }

		    // Render the template
    		include(sprintf("%s/bp-setting.php", dirname(__FILE__)));
		}

    } // END class Bp_Bulk_Group_Importer
}

if(class_exists('Bp_Bulk_Group_Importer'))
{
    // Installation and uninstallation hooks
    register_activation_hook(__FILE__, array('Bp_Bulk_Group_Importer', 'activate'));
    register_deactivation_hook(__FILE__, array('Bp_Bulk_Group_Importer', 'deactivate'));  
    register_uninstall_hook(__FILE__, array('Bp_Bulk_Group_Importer', 'bp_bulk_deinstall'));  

    // instantiate the plugin class
    $Bp_Bulk_Group_Importer = new Bp_Bulk_Group_Importer();
} 